/* Serial LCD support routines */

#define _LCD_PES_c_

#include "LCD_PES.h"
#include    <xc.h>
//#include "pconfig.h"
#include    <I2C_PES.h>
/* local definitions	*/

//static BYTE LCDpins;

extern void DelayMsec(int msec);

BYTE _LCD_build_byte();
void _LCD_write_lower_nibble(BYTE l);
void _LCD_write_upper_nibble(BYTE u);

void LCD_open()
{
    I2C_idle();
    I2C_open(MASTER, SLEW_OFF);           // Begin transfer, claim I2C BUS
    SSPADD = 0x09;
    I2C_idle();
    I2C_start();
    I2C_idle();
    I2C_write(LCD_PIC_I2C_ADDR);          // Tell all I2C devices you are talking to LCD_PIC_I2C_ADDR
}
void LCD_close()
{
    I2C_idle();
    I2C_close();
}

void LCD_init()
{
    LCD_open();

    // Following bytes are all Command bytes, i.e. address = 0x00
    LCD_write_byte(0x00, 0x03);   // Write Nibble 0x03 three times (per HD44780U initialization spec)
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x03);   //
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x03);   //
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x02);   // Write Nibble 0x02 once (per HD44780U initialization spec)
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x02);   // Write Nibble 0x02 once (per HD44780U initialization spec)
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x01);   // Set mode: 4-bit, 2+lines, 5x8 dots
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x0C);   // Display ON 0x0C
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x01);   // Clear display
    DelayMsec(5);                // (per HD44780U initialization spec)
    LCD_write_byte(0x00, 0x06);   // Set cursor to increment
    DelayMsec(5);                // (per HD44780U initialization spec)

//    LCD_close();
}

void LCD_BL(BYTE status)
{
    LCD_BL_Status = status;
    LCD_write_byte(0x00, 0x00);
}

void LCD_goto(BYTE x, BYTE y)   //x=col, y=row
{
BYTE address;

   switch(y)
     {
      case 1:
        address = LCD_PIC_LINE_1_ADDRESS;
        break;

      case 2:
        address = LCD_PIC_LINE_2_ADDRESS;
        break;

      case 3:
        address = LCD_PIC_LINE_3_ADDRESS;
        break;

      case 4:
        address = LCD_PIC_LINE_4_ADDRESS;
        break;

      default:
        address = LCD_PIC_LINE_1_ADDRESS;
        break;
     }

   address += x-1;
   LCD_write_byte(0, 0x80 | address);
}


// LCD initialization by instruction 								*/
// 4-bit 2 line 																		*/
// wait times are set for 8MHz clock (TCY 500ns)
/*
void old_LCD_init( void )
{
//	RCK = 0;
	LCDpins =  0x30;	// send 0x3
        CLR_RW;                 // write
	CLR_RS;                 // command
	SET_E;
	DelayMsec ( 100 );    // wait 100msec
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);

	SET_E;
	DelayMsec(10);    // wait 10ms
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);

        SET_E;					// send 0x3
	DelayMsec(10);
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);

        LCDpins =  0x20;				// send 0x2 - switch to 4-bit
	SET_E; 
	DelayMsec(10);
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);

        // regular transfers start here
	DelayMsec(10);
	LCD_sendcmd ( 0x28 );		//4-bit 2-line 5x7-font
	DelayMsec(10);
	LCD_sendcmd ( LCD_CLRSCR );		//clear display
	DelayMsec(10);
	LCD_sendcmd ( LCD_DISPLAYON );		//turn off cursor, turn on display
	DelayMsec(10);
	LCD_sendcmd ( LCD_CURSORON );		//Increment cursor automatically
	return;
}
*/

/* sends a byte to LCD in 4-bit mode 
void LCD_sendbyte( BYTE tosend )
{
	LCDpins &= 0x0f;				//prepare place for the upper nibble 
	LCDpins |= ( (tosend<<4) & 0xf0 );	//copy upper nibble to LCD variable
	SET_E;							//send
	DelayMsec(10);    // wait 10ms
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);
	LCDpins &= 0x0f;					//prepare place for the lower nibble
	LCDpins |= (tosend) & 0xf0; 	//copy lower nibble to LCD variable
	SET_E;								//send
	DelayMsec(10);    // wait 10ms
	I2C_write_byte(LCDpins);
	CLR_E;
	DelayMsec(1);    // wait 1ms
	I2C_write_byte(LCDpins);
}

// send NULL-terminated string
void LCD_send_string( const char *str_ptr )
{
	while (*str_ptr) {
		LCD_sendchar(*str_ptr);
		str_ptr++;
	}
}

// sends a NULL-terminated RAM string to sendchar()
void LCD_send_ram_string (char *str_ptr)
{
	while (*str_ptr) {
		LCD_sendchar(*str_ptr);
		str_ptr++;
	}
}

// sending 2 ASCII symbols representing input byte in hex
void LCD_send_hexbyte ( BYTE data )
{
	BYTE temp = data>>4;			//prepare first output character

	if ( temp > 9 ) temp+=7;		//jump to letters in ASCII table
	LCD_sendchar ( temp + 0x30 );
	
	data = data & 0x0f;				// mask 4 high bits
	if ( data > 9 ) data+=7;
	LCD_sendchar ( data + 0x30 );	
}

void LCD_second_row( void )
{
	LCD_sendcmd( 0xc0 );
}

void LCD_home( void )
{
	LCD_sendcmd( LCD_CLRSCR );
	DelayMsec(10);
	LCD_sendcmd( LCD_HOME );
}
*/

void LCD_write_string(const char *str)
{
   // Writes a string text[] to LCD via I2C
   pin_RS  = 1;
   pin_RW  = 0;
   pin_E   = 0;
   pin_BL  = LCD_BL_Status;

   while (*str)
   {
        // Send upper nibble
        _LCD_write_upper_nibble(*str);

        // Send lower nibble
        _LCD_write_lower_nibble(*str);

        str++;
   }
}
void LCD_write_byte(BYTE address, BYTE n)
{
    if (address)
    {
        pin_RS=1;   // Data
    }
    else
    {
        pin_RS=0;   // Command
    }

    pin_RW  = 0;
    pin_E   = 0;
    pin_BL  = LCD_BL_Status;

    // Send upper nibble
   _LCD_write_upper_nibble(n);

    // Send lower nibble
   _LCD_write_lower_nibble(n);
}

void LCD_clear()
{
    LCD_write_byte(0x00,0x01);
    DelayMsec(5);
}

void LCD_clear_line(BYTE line)
{
    LCD_goto(1,line);
    for (int i = 0; i<20; i++)
    {
        LCD_write_string(" ");
    }
    LCD_goto(1,line);
}

void _LCD_write_upper_nibble(BYTE u)
{
    // Send upper nibble
    if(bit_test(u,7))
        pin_D7=1;
    else
        pin_D7=0;

    if(bit_test(u,6))
        pin_D6=1;
    else
        pin_D6=0;

    if(bit_test(u,5))
        pin_D5=1;
    else
        pin_D5=0;

    if(bit_test(u,4))
        pin_D4=1;
    else
        pin_D4=0;

   pin_E = 0;
   I2C_write(_LCD_build_byte());
   pin_E = 1;
   I2C_write(_LCD_build_byte());
   pin_E = 0;
   I2C_write(_LCD_build_byte());
}

void _LCD_write_lower_nibble(BYTE l)
{
    // Send lower nibble
    if(bit_test(l,3))
        pin_D7=1;
    else
        pin_D7=0;

    if(bit_test(l,2))
        pin_D6=1;
    else
        pin_D6=0;

    if(bit_test(l,1))
        pin_D5=1;
    else
        pin_D5=0;

    if(bit_test(l,0))
        pin_D4=1;
    else
        pin_D4=0;

    pin_E = 0;
    I2C_write(_LCD_build_byte());
    pin_E = 1;
    I2C_write(_LCD_build_byte());
    pin_E = 0;
    I2C_write(_LCD_build_byte());
}

BYTE _LCD_build_byte()
{
    BYTE ret = 0x00;

    ret |= pin_E    << 2;
    ret |= pin_RW   << 1;
    ret |= pin_RS   << 0;
    ret |= pin_D4   << 4;
    ret |= pin_D5   << 5;
    ret |= pin_D6   << 6;
    ret |= pin_D7   << 7;
    ret |= pin_BL   << 3;

    return ret;
}
