/* 
 * File:   TigTac_1825_Main.c
 * Author: Paul E. Schoen
 *
 * Created on January 28, 2015, 11:43 PM
 * Ver 0.1 - 0x91c7 - 2/3/15 - First working version
 */
#define Version         "0.2"

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
//#include <lcd.h>
#include <Includes_PES.h>
#include "LCD_PES.h"
#include <i2c.h>

// Configuration word for PIC16F877
#ifdef  _16F877
__CONFIG( FOSC_HS & WDTE_OFF & PWRTE_ON & CP_OFF & BOREN_ON
		& LVP_OFF & CPD_OFF & WRT_ON & DEBUG_OFF);
#else
#ifdef  _16F1825
__CONFIG( FOSC_HS & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF & BOREN_ON
		& LVP_OFF & CPD_OFF & DEBUG_OFF);
#pragma config WRT = ALL
#endif
#endif


#define USE_OR_MASKS

#define	ADCON0SET	0b00001101	//<6:2>CHSEL: 00011=AN3, <1>GO/DONE, <0>ADON

#define	ADCON1SET	0b11010011	//<7>ADFM 1=right, <6:4>ADCS 101=Fosc/16, <3:2>N/A, <1:0>ADPREF 11=FVR

#define	ANSELASET	0b00010000	//<0,1,2,x,4>=ANSA0, ANSA1, ANSA2, ANSA4
                                        // RA4(AN3=ANSA4) Analog (Vin)
					//RA4=AN3, RA2=AN2, RA1=AN1, RA0=AN0
#define	ANSELCSET	0b00000000	//<0:3>=ANSC0, ANSC1, ANSC2, ANSC4

#define	FVRCONSET	0b10000011	//<7>FVREN, <6>FVRRDY, <5>TSEN, <4>TSRNG, <3:2>CDAFVR, <1:0>10=2.048, 11=4.096

#define	OPTIONSET	0b10001000	//<7>WPU, <6>INTEDG, <5>TMR0CS 0=Fosc/4, <4> TMR0SE <3>PSA 0=TMR0,
					//<2:0>PS 000=1:2, 011=1:16, 111=1:256

#define	OSCCONSET	0b01111010	//<7>SPLLEN, <6:3>IRCF 1011=1MHz, <2>N/A, <1:0>1x=IntOsc
					// 1011=1 MHz, 1101=4 MHz, 1111=16 MHz

#define T1CONSET	0b00000000      // <7:6>CS: 00-fOsc/4, 00=Inst Cycle fOsc/4
                                        // <5:4>Prescale: 00=1, <3>1=T1OSCEN, <2> T1SYNC, <1>N/A, <0>1=ON
                                        // 16 MHz clock/4: 4000000 /65536 = 61.035 Hz = 16.384 mSec

#define	T2CONSET	0b00000010	// <6:3> Postscale 1, <0:1> Prescale 00=1 01=4 10=16 11=64
#define PS              16               //Prescaler

#define	TRISASET	0b00011111	// Set <0:4> as input, <5> as output
//#define	TRISBSET	0b00011111	// Set <0:4> as input, <5> as output
#define	TRISCSET	0b00110011	// Set <0:1,4:5> as input, <2:3> as output
#define	INTCONSET	0b00000000	// Setup INTCON 7:GEIE, 6:PEIE, 5:TMR0IE, 4:INTE, 3:IOCIE, 2:TMR0IF, 1:INTF, 0:IOCIF
#define	PIE1SET		0b00000000	// <7>TMR1GIE, <6>ADIE, <5>RCIE, <4>TXIE, <3>SSP1IE, <2>CCP1IE, <1>TMR2IE, <0>TMR1IE
#define	PIR1SET		0b00000000	// <7>TMR1GIF, <6>ADIF, <5>RCIF, <4>TXIF, <3>SSP1IF, <2>CCP1IF, <1>TMR2IF, <0>TMR1IF

#define TM0VAL          19              // reload value for 50 uSec > 1 mSec

#define RED_OFF          LATC |= 0b00000100
#define RED_ON           LATC &= ~0b00000100
#define GRN_OFF          LATC |= 0b00001000
#define GRN_ON           LATC &= ~0b00001000

volatile unsigned long int   timer1=0;		// running time in 1000 uSec steps
volatile unsigned int   timer0=0;		// delay time in 1.0 mSec steps
volatile unsigned char  tm0=TM0VAL;             // time in 50 uSec steps
volatile unsigned int   dwell=0;
volatile unsigned long int   rotation=0;

void    Initialize(void);
void    SetupTimer0();
void    DelayMsec(int msec);

void    interrupt   HighIntCode()           // for XC8 compiler
    {
    if( INTCONbits.TMR0IF ) {               // 16 MHz, 0.25 uSec update of TMR0, PS=N/A, 200 counts = 50 uSec
        TMR0 = 256-200+13;                 // 50 uSec
        if( tm0 > 0)
            tm0--;
        else {
            tm0=19;
            if(timer0>0)
                timer0--;
            timer1++;
            }
        INTCONbits.TMR0IF = 0;
    	}
    else if( INTCONbits.IOCIF ) {
        if(IOCAFbits.IOCAF3) {
            if(PORTAbits.RA3 == 1) {  //switch opened
                GRN_OFF; RED_ON;
                dwell = timer1;
                timer1 = 0; }
            else if(PORTAbits.RA3 == 0) {  //switch closed
                GRN_ON; RED_OFF;
                rotation = timer1+dwell;
                timer1 = 0; }
            IOCAFbits.IOCAF3 = 0;
            }
        INTCONbits.IOCIF = 0;
        }

    }//

int main(int argc, char** argv) {
    char line1[16];
    char line2[16];
    Initialize();
    DelayMsec(1000);
    SetupTimer0();
    I2C_init();      // in I2C_PES
    LCD_open();   // causes errors
    LCD_init();     // in LCD_PES
    LCD_clear();
    LCD_goto(1,1);
    LCD_write_string( " P S Technology " );
    LCD_goto(1,2);
    sprintf(line2, " TigTac Ver %s", Version);
    LCD_write_string( line2 );
    DelayMsec(2000);
    while(PORTAbits.RA3==1); //wait for start
    LCD_clear();
    while(1) {
      LCD_goto(1,1);
      sprintf(line1, "Dwell: %04d mS  ", dwell);
      LCD_write_string( line1 );
      LCD_goto(1,2);
      sprintf(line2, "RPM: %05d      ", 60000/rotation);
      LCD_write_string( line2 );
      DelayMsec(300); }
//    return (EXIT_SUCCESS);
}

void    Initialize(void)    {
    LATA = 0;
    OSCCON = OSCCONSET;
    OPTION_REG = OPTIONSET;
    INTCON = INTCONSET;
    TRISA = TRISASET;
    TRISC = TRISCSET;
    LATC = 0;
    ANSELA = ANSELASET;         //Only AN4 analog
    ANSELC = ANSELCSET;         //All digital
    IOCAN = 0b00001000;        //AN3 interrupt on change negative
    IOCAP = 0b00001000;        //AN3 interrupt on change negative
    INTCONbits.IOCIE = 1;       // Enable IOC
}

void    SetupTimer0()
{
    INTCONbits.GIE = 0;		//Turn off Global Interrupts
    OPTION_REGbits.PSA = 1;	//0 = assigned to prescaler
    OPTION_REGbits.PS2 = 0;	//Prescale
    OPTION_REGbits.PS1 = 0;	//Prescale
    OPTION_REGbits.PS0 = 0;	//Prescale
    INTCONbits.TMR0IE = 1;
    INTCONbits.PEIE = 1;	//Turn on Peripheral Interrupts
    INTCONbits.GIE = 1;		//Turn on Global Interrupts
}

void    DelayMsec(int msec)
    {
    if(INTCONbits.TMR0IE) {
        timer0=msec;
        while(timer0>0)
            asm("NOP");
        }
    }


